
class APIS(object):
    MODES = 'https://www.hermchats.com/gw/chatweb/gpt/modes'
    WSS='wss://www.hermchats.com/gw/chatgpt/ws/{}'
    DIALOGS='https://www.hermchats.com/gw/chatweb/gpt/dialogs'
    CREATE_DIALOG='https://www.hermchats.com/gw/chatweb/gpt/createDialog'


HEADERS = {
    'accept': 'application/json, text/plain, */*',
    'accept-language': 'en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7',
    'cache-control': 'no-cache',
    'content-type': 'application/json',
    'dnt': '1',
    'noninductive': 'true',
    'origin': 'https://www.hermchats.com',
    'pragma': 'no-cache',
    'priority': 'u=1, i',
    'referer': 'https://www.hermchats.com/chat/MTE0NTEK',
    'sec-ch-ua': '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
    'sec-ch-ua-mobile': '?0',
    'sec-ch-ua-platform': '"macOS"',
    'sec-fetch-dest': 'empty',
    'sec-fetch-mode': 'cors',
    'sec-fetch-site': 'same-origin',
    'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36',
}



import asyncio
import json
import typing
from uuid import uuid4

import httpx
import pydantic
import websockets
from loguru import logger

from .configs import APIS, HEADERS
from .types import Mode
from .models import models

def get_headers(token: str):
    headers = HEADERS
    headers['jtoken'] = token
    return headers

class Juchats(object):
    _modes = None

    def __init__(self, token: str, model: str = "gpt-4o-2024-05-13"):
        self.token = token
        self.model = model
        self._header = get_headers(token)
        self._model_id = None
        self._dialog_id = None

    async def __aenter__(self):
        self._modes = [Mode(**x) for x in models]
        for mode in self._modes:
            if mode.name == self.model:
                self._model_id = mode.id
                break
        else:
            self.help_with_modes()
            raise ValueError(f"Model {self.model} not found")

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    def help_with_modes(self):
        print("Available modes are:")
        for mode in self._modes:
            print(f"{mode.id}: {mode.name}")

    async def modes(self) -> typing.List[Mode]:
        async with httpx.AsyncClient() as client:
            response = await client.get(APIS.MODES, headers=self._header)
            data = response.json()['data']
            modes = []
            for item in data:
                modes.extend([Mode(**x) for x in item['modes']])
            return modes

    async def get_dialog_id(self) -> int:
        async with httpx.AsyncClient() as client:
            response = await client.post(APIS.DIALOGS,
                                         headers=self._header,
                                         json={})
            response.raise_for_status()
            data = response.json()['data']
            for x in data:
                if x['modeId'] == self._model_id:
                    return x['id']

            logger.info('create new dialog for model {}'.format(self.model))
            _type = await self.get_type()
            response = await client.post(APIS.CREATE_DIALOG,
                                         headers=self._header,
                                         json={
                                             'dialogType': 1,
                                             'name': self.model,
                                             'type': _type,
                                             'ttsLanguageTypeId': 0,
                                             'ttsType': 0,
                                             'modeId': self._model_id,
                                             'contextId': '',
                                         })
            response.raise_for_status()
            return int(response.json()['data'])

    async def get_model_id(self) -> int:
        return self._model_id

    async def get_type(self) -> int:
        return 10

    async def chat(
        self,
        query: str,
        show_stream: bool = False,
    ):
        dialog_id = await self.get_dialog_id()
        model_id = await self.get_model_id()
        _type = await self.get_type()
        async with websockets.connect(APIS.WSS.format(self.token),
                                      extra_headers=self._header) as ws:
            message = {
                "contextId": '',
                "dialogId": dialog_id,
                "event": 1,
                "fileUuid": "",
                "languageTypeId": 0,
                "modeId": model_id,
                "prompt": query,
                "requestId": str(uuid4()),
                "type": _type,
            }

            await ws.send(json.dumps(message))

            try:
                text = ''
                while True:
                    response = await ws.recv()
                    if '[DONE]' in response:
                        return text
                    js = json.loads(response)
                    content = js.get('data', {}).get('content')
                    if content:
                        text += content
                        if show_stream:
                            print(content, end="", flush=True)
                    if int(js.get('code', 200)) != 200:
                        logger.info(js)
                        return text
            except websockets.ConnectionClosed:
                logger.error("Connection closed by the server.")
            except Exception as e:
                logger.error(f"Error occurred: {e}")



models = [{
    "id": 5,
    "type": 7,
    "name": "claude-3-haiku-20240307",
    "showName": "Claude Mezzo \r\n",
    "maxToken": 180000,
    "searchFlag": 0,
    "remark": "混合模型，根据对话自动选择适当的 Claude3 模型，当前预览版，支持自动联网及识别对话中的图像\r\n"
}, {
    "id": 6,
    "type": 6,
    "name": "claude-3-opus-20240229",
    "showName": "Claude3 Opus",
    "maxToken": 180000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 7,
    "type": 5,
    "name": "mistralai/mixtral-8x22b-instruct",
    "showName": "Mixtral Forte\r\n",
    "maxToken": 30768,
    "searchFlag": 0,
    "remark": "根据对话自动选择适当的Mixtral模型，当前预览版，支持对话、自动联网，建议使用英文对话"
}, {
    "id": 9,
    "type": 2,
    "name": "gpt-4-turbo-2024-04-09",
    "showName": "GPT Mezzo\r\n",
    "maxToken": 118000,
    "searchFlag": 0,
    "remark": "联网模型，包含最新的 GPT-4o 及 自动联网"
}, {
    "id": 10,
    "type": 0,
    "name": "gpt-4-turbo-2024-04-09",
    "showName": "GPT Forte\r\n",
    "maxToken": 118000,
    "searchFlag": 0,
    "remark": "混合模型，根据对话自动选择 GPT4 或 GPT-4o 模型，当前预览版，支持对话、自动联网、图像识别及生成"
}, {
    "id": 11,
    "type": 3,
    "name": "dall-e-3",
    "showName": "DALL · E3",
    "maxToken": 4096,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 12,
    "type": 8,
    "name": "meta-llama/llama-3-70b-instruct",
    "showName": "Llama3 70B",
    "maxToken": 30768,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 13,
    "type": 9,
    "name": "google/gemini-pro-1.5",
    "showName": "Gemini 1.5 Pro",
    "maxToken": 1000000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 14,
    "type": 10,
    "name": "deepseek-chat",
    "showName": "Deepseek",
    "maxToken": 1180000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 15,
    "type": 9,
    "name": "google/gemini-flash-1.5",
    "showName": "Gemini-flash",
    "maxToken": 100000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 16,
    "type": 11,
    "name": "gpt-4o-2024-05-13",
    "showName": "GPT4o",
    "maxToken": 128000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 17,
    "type": 12,
    "name": "claude-3-opus-20240229",
    "showName": "Claude3 Opus(100K)",
    "maxToken": 90000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 18,
    "type": 13,
    "name": "Stable Image Ultra",
    "showName": "Stable Diffusion 3 Ultra",
    "maxToken": 10000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 19,
    "type": 15,
    "name": "claude-3-5-sonnet-20240620",
    "showName": "Claude 3.5 Sonnet",
    "maxToken": 180000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 20,
    "type": 16,
    "name": "gpt-4o-mini-2024-07-18",
    "showName": "GPT4o-mini",
    "maxToken": 118000,
    "searchFlag": 0,
    "remark": None
}, {
    "id": 21,
    "type": 17,
    "name": "meta-llama/llama-3.1-405b-instruct",
    "showName": "Llama3.1 405B",
    "maxToken": 121072,
    "searchFlag": 0,
    "remark": None
}]
